import 'package:tingyun_flutter_plugin/src/agent/util/string_utils.dart';

import 'log_level.dart';

const bool _DEFAULT_REPORT_CRASH = false;
const bool _DEFAULT_MONITOR_WEB_REQUEST = false;
const bool _DEFAULT_UI_ENABLED = false;
const bool _DEFAULT_USETACTION_ENABLED = false;
const bool _DEFAULT_CERTIFICATE_VALIDATION = false;
const bool _DEFAULT_USER_OPT_IN = false;

const LogLevel _DEFAULT_LOG_LEVEL = LogLevel.Info;

/// Configuration class which can be used for manual instrumentation.
class Configuration{

  final LogLevel _logLevel;
  final bool _reportCrash;
  final bool _monitorWebRequest;
  final String _beaconUrl;
  final String _applicationId;
  final bool _certificateValidation;
  final bool _userOptIn;
  final bool _uiEnabled;
  final bool _actionEnabled;

  /// Constructor of configuration class.
  const Configuration(
      {bool reportCrash = _DEFAULT_REPORT_CRASH,
      bool monitorWebRequest = _DEFAULT_MONITOR_WEB_REQUEST,
      LogLevel logLevel = _DEFAULT_LOG_LEVEL,
        bool uiEnabled = _DEFAULT_UI_ENABLED,
        bool userActionEnabled = _DEFAULT_USETACTION_ENABLED,
      String beaconUrl = "",
      String applicationId = "",
      bool certificateValidation = _DEFAULT_CERTIFICATE_VALIDATION,
      bool userOptIn = _DEFAULT_USER_OPT_IN})
      : _reportCrash = reportCrash,
        _monitorWebRequest = monitorWebRequest,
        _uiEnabled = uiEnabled,
        _actionEnabled = userActionEnabled,
        _logLevel = logLevel,
        _applicationId = applicationId,
        _beaconUrl = beaconUrl,
        _certificateValidation = certificateValidation,
        _userOptIn = userOptIn;

  /// Returns if crash reporting is enabled.
  bool get isCrashReportingEnabled {
    return _reportCrash;
  }

  bool get uiEnabled => _uiEnabled;

  bool get actionEnabled => _actionEnabled;

  /// Check if the properties for the Autostart are available and valid
  /// If [beaconUrl] and [applicationId] are available this will return true
  bool get isAutoStartPropertyAvailable {
    return !StringUtils.isStringNullEmptyOrWhitespace(_beaconUrl) &&
        !StringUtils.isStringNullEmptyOrWhitespace(_applicationId);
  }

  /// Returns if web request monitoring is enabled
  bool get isWebRequestMonitoringEnabled => _monitorWebRequest;
  LogLevel get logLevel => _logLevel;

  /// Generating the Startup configuration for a manual startup. Only
  /// relevant configurations will be passed to the native agents.
  /// Currently this is applicationId, beaconUrl and loglevel.
  Map<String, dynamic> getStartupConfiguration() {
    Map<String, dynamic> startupConfiguration = Map();

    if (!StringUtils.isStringNullEmptyOrWhitespace(_applicationId)) {
      startupConfiguration["applicationId"] = _applicationId;
    }

    if (!StringUtils.isStringNullEmptyOrWhitespace(_beaconUrl)) {
      startupConfiguration["beaconUrl"] = _beaconUrl;
    }

    //这里应该说的默认是info，如果是debug的话，才需要放到map中
    if(_logLevel == LogLevel.Debug){
      // Only sending the non-default value which is debug
      startupConfiguration["logLevel"] = "debug";
    }

    if (!_certificateValidation) {
      // Only sending the non-default value which is false
      startupConfiguration["certificateValidation"] = false;
    }

    if (_userOptIn) {
      // Only sending the non-default value which is true
      startupConfiguration["userOptIn"] = true;
    }

    //默认是抓取crash的
    if (!_reportCrash) {
      // Only sending the non-default value which is false
      startupConfiguration["crashReporting"] = false;
    }

    return startupConfiguration;
  }


}